@@ -92,6 +92,57 @@ static void print_vpf(enum sms_validity_period_format vpf,
 	}
 }
 
+static void dump_details(struct sms *sms)
+{
+	if (sms->sc_addr.address[0] == '\0')
+		g_print("SMSC Address absent, default will be used\n");
+	else
+		g_print("SMSC Address number_type: %d, number_plan: %d, %s\n",
+			(int)sms->sc_addr.number_type,
+			(int)sms->sc_addr.numbering_plan, sms->sc_addr.address);
+
+	switch (sms->type) {
+	case SMS_TYPE_DELIVER:
+		g_print("Type: Deliver\n");
+
+		g_print("Originator-Address: %d, %d, %s\n",
+			(int)sms->deliver.oaddr.number_type,
+			(int)sms->deliver.oaddr.numbering_plan,
+			sms->deliver.oaddr.address);
+
+		g_print("PID: %d\n", (int)sms->deliver.pid);
+		g_print("DCS: %d\n", (int)sms->deliver.dcs);
+
+		print_scts(&sms->deliver.scts, "Timestamp");
+
+		break;
+	case SMS_TYPE_SUBMIT:
+		g_print("Type: Submit\n");
+
+		g_print("Message Reference: %u\n", (int)sms->submit.mr);
+
+		g_print("Destination-Address: %d, %d, %s\n",
+			(int)sms->submit.daddr.number_type,
+			(int)sms->submit.daddr.numbering_plan,
+			sms->submit.daddr.address);
+
+		g_print("PID: %d\n", (int)sms->submit.pid);
+		g_print("DCS: %d\n", (int)sms->submit.dcs);
+
+		print_vpf(sms->submit.vpf, &sms->submit.vp);
+
+		break;
+	case SMS_TYPE_STATUS_REPORT:
+		break;
+	case SMS_TYPE_COMMAND:
+	case SMS_TYPE_DELIVER_REPORT_ACK:
+	case SMS_TYPE_DELIVER_REPORT_ERROR:
+	case SMS_TYPE_SUBMIT_REPORT_ACK:
+	case SMS_TYPE_SUBMIT_REPORT_ERROR:
+		break;
+	}
+}
+
 static void test_simple_deliver()
 {
 	struct sms sms;
@@ -114,23 +165,8 @@ static void test_simple_deliver()
 	g_assert(ret);
 	g_assert(sms.type == SMS_TYPE_DELIVER);
 
-	if (g_test_verbose()) {
-		g_print("SMSC Address number_type: %d, number_plan: %d, %s\n",
-			(int)sms.sc_addr.number_type,
-			(int)sms.sc_addr.numbering_plan, sms.sc_addr.address);
-
-		g_print("SMS type: %d\n", (int)sms.type);
-
-		g_print("Originator-Address: %d, %d, %s\n",
-			(int)sms.deliver.oaddr.number_type,
-			(int)sms.deliver.oaddr.numbering_plan,
-			sms.deliver.oaddr.address);
-
-		g_print("PID: %d\n", (int)sms.deliver.pid);
-		g_print("DCS: %d\n", (int)sms.deliver.dcs);
-
-		print_scts(&sms.deliver.scts, "Timezone");
-	}
+	if (g_test_verbose())
+		dump_details(&sms);
 
 	g_assert(sms.sc_addr.number_type == SMS_NUMBER_TYPE_INTERNATIONAL);
 	g_assert(sms.sc_addr.numbering_plan == SMS_NUMBERING_PLAN_ISDN);
@@ -200,23 +236,8 @@ static void test_alnum_sender()
 	g_assert(ret);
 	g_assert(sms.type == SMS_TYPE_DELIVER);
 
-	if (g_test_verbose()) {
-		g_print("SMSC Address number_type: %d, number_plan: %d, %s\n",
-			(int)sms.sc_addr.number_type,
-			(int)sms.sc_addr.numbering_plan, sms.sc_addr.address);
-
-		g_print("SMS type: %d\n", (int)sms.type);
-
-		g_print("Originator-Address: %d, %d, %s\n",
-			(int)sms.deliver.oaddr.number_type,
-			(int)sms.deliver.oaddr.numbering_plan,
-			sms.deliver.oaddr.address);
-
-		g_print("PID: %d\n", (int)sms.deliver.pid);
-		g_print("DCS: %d\n", (int)sms.deliver.dcs);
-
-		print_scts(&sms.deliver.scts, "Timestamp");
-	}
+	if (g_test_verbose())
+		dump_details(&sms);
 
 	g_assert(sms.sc_addr.number_type == SMS_NUMBER_TYPE_INTERNATIONAL);
 	g_assert(sms.sc_addr.numbering_plan == SMS_NUMBERING_PLAN_ISDN);
@@ -263,6 +284,7 @@ static void test_alnum_sender()
 
 	g_free(utf8);
 }
+
 static void test_deliver_encode()
 {
 	struct sms sms;
@@ -361,30 +383,8 @@ static void test_simple_submit()
 	g_assert(ret);
 	g_assert(sms.type == SMS_TYPE_SUBMIT);
 
-	if (g_test_verbose()) {
-		if (sms.sc_addr.address[0] == '\0')
-			g_print("SMSC Address absent, default will be used\n");
-		else
-			g_print("SMSC Address number_type: %d,"
-				" number_plan: %d, %s\n",
-				(int)sms.sc_addr.number_type,
-				(int)sms.sc_addr.numbering_plan,
-				sms.sc_addr.address);
-
-		g_print("SMS type: %d\n", (int)sms.type);
-
-		g_print("Message Reference: %u\n", (int)sms.submit.mr);
-
-		g_print("Destination-Address: %d, %d, %s\n",
-			(int)sms.submit.daddr.number_type,
-			(int)sms.submit.daddr.numbering_plan,
-			sms.submit.daddr.address);
-
-		g_print("PID: %d\n", (int)sms.submit.pid);
-		g_print("DCS: %d\n", (int)sms.submit.dcs);
-
-		print_vpf(sms.submit.vpf, &sms.submit.vp);
-	}
+	if (g_test_verbose())
+		dump_details(&sms);
 
 	g_assert(strlen(sms.sc_addr.address) == 0);
 
@@ -471,102 +471,202 @@ static void test_submit_encode()
 	g_free(encoded_pdu);
 }
 
-static const char *header_test = "0041000B915121551532F40000631A0A031906200A03"
-	"2104100A032705040A032E05080A043807002B8ACD29A85D9ECFC3E7F21C340EBB41E"
-	"3B79B1E4EBB41697A989D1EB340E2379BCC02B1C3F27399059AB7C36C3628EC2683C6"
-	"6FF65B5E2683E8653C1D";
-static int header_test_len = 100;
-static const char *header_test_expected = "EMS messages can contain italic, bold"
-	", large, small and colored text";
+struct text_format_header {
+	unsigned char len;
+	unsigned char start;
+	unsigned char span;
+	unsigned char format;
+	unsigned char color;
+};
 
-static void test_udh_iter()
+struct ems_udh_test {
+	const char *pdu;
+	unsigned int len;
+	const char *expected;
+	unsigned int udl;
+	unsigned int udhl;
+	unsigned int data_len;
+	struct text_format_header formats[];
+};
+
+static struct ems_udh_test ems_udh_test_1 = {
+	.pdu = "0041000B915121551532F40000631A0A031906200A032104100A03270504"
+		"0A032E05080A043807002B8ACD29A85D9ECFC3E7F21C340EBB41E3B79B1"
+		"E4EBB41697A989D1EB340E2379BCC02B1C3F27399059AB7C36C3628EC26"
+		"83C66FF65B5E2683E8653C1D",
+	.len = 100,
+	.expected = "EMS messages can contain italic, bold, large, small and"
+		" colored text",
+	.formats = {
+		{
+			.len = 3,
+			.start = 0x19,
+			.span = 0x06,
+			.format = 0x20,
+		},
+		{
+			.len = 3,
+			.start = 0x21,
+			.span = 0x04,
+			.format = 0x10,
+		},
+		{
+			.len = 3,
+			.start = 0x27,
+			.span = 0x05,
+			.format = 0x04,
+		},
+		{
+			.len = 3,
+			.start = 0x2E,
+			.span = 0x05,
+			.format = 0x08,
+		},
+		{
+			.len = 4,
+			.start = 0x38,
+			.span = 0x07,
+			.format = 0x00,
+			.color = 0x2B,
+		},
+		{
+			.len = 0,
+		}
+	},
+	.udl = 99,
+	.udhl = 26,
+	.data_len = 87,
+};
+
+static struct ems_udh_test ems_udh_test_2 = {
+	.pdu = "079194712272303351030B915121340195F60000FF80230A030F07230A031"
+		"806130A031E0A430A032E0D830A033D14020A035104F60A0355010600159"
+		"D9E83D2735018442FCFE98A243DCC4E97C92C90F8CD26B3407537B92C67A"
+		"7DD65320B1476934173BA3CBD2ED3D1F277FD8C76299CEF3B280C92A7CF6"
+		"83A28CC4E9FDD6532E8FE96935D",
+	.len = 126,
+	.expected = "This is a test\nItalied, bold, underlined, and "
+		"strikethrough.\nNow a right aligned word.",
+	.formats = {
+		{
+			.len = 3,
+			.start = 0x0f,
+			.span = 0x07,
+			.format = 0x23,
+		},
+		{
+			.len = 3,
+			.start = 0x18,
+			.span = 0x06,
+			.format = 0x13,
+		},
+		{
+			.len = 3,
+			.start = 0x1e,
+			.span = 0x0a,
+			.format = 0x43,
+		},
+		{
+			.len = 3,
+			.start = 0x2e,
+			.span = 0x0d,
+			.format = 0x83,
+		},
+		{
+			.len = 3,
+			.start = 0x3d,
+			.span = 0x14,
+			.format = 0x02,
+		},
+		{
+			.len = 3,
+			.start = 0x51,
+			.span = 0x04,
+			.format = 0xf6,
+		},
+		{
+			.len = 3,
+			.start = 0x55,
+			.span = 0x01,
+			.format = 0x06,
+		},
+	},
+	.udl = 128,
+	.udhl = 35,
+	.data_len = 112,
+};
+
+static void test_ems_udh(gconstpointer data)
 {
+	const struct ems_udh_test *test = data;
 	struct sms sms;
 	unsigned char *decoded_pdu;
 	long pdu_len;
 	gboolean ret;
-	int data_len;
-	int udhl;
+	unsigned int data_len;
+	unsigned int udhl;
 	struct sms_udh_iter iter;
 	int max_chars;
 	unsigned char *unpacked;
 	char *utf8;
+	int i;
 
-	decoded_pdu = decode_hex(header_test, -1, &pdu_len, 0);
+	decoded_pdu = decode_hex(test->pdu, -1, &pdu_len, 0);
 
 	g_assert(decoded_pdu);
-	g_assert(pdu_len == (long)strlen(header_test) / 2);
+	g_assert(pdu_len == (long)strlen(test->pdu) / 2);
 
-	ret = sms_decode(decoded_pdu, pdu_len, TRUE,
-				header_test_len, &sms);
+	ret = sms_decode(decoded_pdu, pdu_len, TRUE, test->len, &sms);
 
 	g_free(decoded_pdu);
 
 	g_assert(ret);
 	g_assert(sms.type == SMS_TYPE_SUBMIT);
 
-	if (g_test_verbose()) {
-		if (sms.sc_addr.address[0] == '\0')
-			g_print("SMSC Address absent, default will be used\n");
-		else
-			g_print("SMSC Address number_type: %d,"
-				" number_plan: %d, %s\n",
-				(int)sms.sc_addr.number_type,
-				(int)sms.sc_addr.numbering_plan,
-				sms.sc_addr.address);
-
-		g_print("SMS type: %d\n", (int)sms.type);
-
-		g_print("Message Reference: %u\n", (int)sms.submit.mr);
-
-		g_print("Destination-Address: %d, %d, %s\n",
-			(int)sms.submit.daddr.number_type,
-			(int)sms.submit.daddr.numbering_plan,
-			sms.submit.daddr.address);
-
-		g_print("PID: %d\n", (int)sms.submit.pid);
-		g_print("DCS: %d\n", (int)sms.submit.dcs);
-
-		print_vpf(sms.submit.vpf, &sms.submit.vp);
-	}
-
+	if (g_test_verbose())
+		dump_details(&sms);
 	udhl = sms.submit.ud[0];
-	g_assert(sms.submit.udl == 99);
-	g_assert(udhl == 26);
+
+	g_assert(sms.submit.udl == test->udl);
+	g_assert(udhl == test->udhl);
 
 	ret = sms_udh_iter_init(&sms, &iter);
 
 	g_assert(ret);
 
-	g_assert(sms_udh_iter_get_ie_type(&iter) == SMS_IEI_TEXT_FORMAT);
-	g_assert(sms_udh_iter_get_ie_length(&iter) == 3);
-	g_assert(sms_udh_iter_has_next(&iter) == TRUE);
-	g_assert(sms_udh_iter_next(&iter) == TRUE);
+	for (i = 0; test->formats[i].len; i++) {
+		if (g_test_verbose()) {
+			int j;
+			unsigned char data[4];
 
-	g_assert(sms_udh_iter_get_ie_type(&iter) == SMS_IEI_TEXT_FORMAT);
-	g_assert(sms_udh_iter_get_ie_length(&iter) == 3);
-	g_assert(sms_udh_iter_has_next(&iter) == TRUE);
-	g_assert(sms_udh_iter_next(&iter) == TRUE);
+			sms_udh_iter_get_ie_data(&iter, data);
 
-	g_assert(sms_udh_iter_get_ie_type(&iter) == SMS_IEI_TEXT_FORMAT);
-	g_assert(sms_udh_iter_get_ie_length(&iter) == 3);
-	g_assert(sms_udh_iter_has_next(&iter) == TRUE);
-	g_assert(sms_udh_iter_next(&iter) == TRUE);
+			g_print("Header:\n");
+			for (j = 0; j < sms_udh_iter_get_ie_length(&iter); j++)
+				g_print("0x%02x ", data[j]);
 
-	g_assert(sms_udh_iter_get_ie_type(&iter) == SMS_IEI_TEXT_FORMAT);
-	g_assert(sms_udh_iter_get_ie_length(&iter) == 3);
-	g_assert(sms_udh_iter_has_next(&iter) == TRUE);
-	g_assert(sms_udh_iter_next(&iter) == TRUE);
+			g_print("\n");
+		}
 
-	g_assert(sms_udh_iter_get_ie_type(&iter) == SMS_IEI_TEXT_FORMAT);
-	g_assert(sms_udh_iter_get_ie_length(&iter) == 4);
-	g_assert(sms_udh_iter_has_next(&iter) == FALSE);
-	g_assert(sms_udh_iter_next(&iter) == FALSE);
-	g_assert(sms_udh_iter_get_ie_type(&iter) == SMS_IEI_INVALID);
+		g_assert(sms_udh_iter_get_ie_type(&iter) ==
+				SMS_IEI_TEXT_FORMAT);
+		g_assert(sms_udh_iter_get_ie_length(&iter) ==
+				test->formats[i].len);
+
+		if (test->formats[i+1].len) {
+			g_assert(sms_udh_iter_has_next(&iter) == TRUE);
+			g_assert(sms_udh_iter_next(&iter) == TRUE);
+		} else {
+			g_assert(sms_udh_iter_has_next(&iter) == FALSE);
+			g_assert(sms_udh_iter_next(&iter) == FALSE);
+			g_assert(sms_udh_iter_get_ie_type(&iter) ==
+					SMS_IEI_INVALID);
+		}
+	}
 
 	data_len = sms_udl_in_bytes(sms.submit.udl, sms.submit.dcs);
 
-	g_assert(data_len == 87);
+	g_assert(data_len == test->data_len);
 
 	max_chars = (data_len - (udhl + 1)) * 8 / 7;
 
@@ -584,7 +684,7 @@ static void test_udh_iter()
 	if (g_test_verbose())
 		g_print("Decoded user data is: %s\n", utf8);
 
-	g_assert(strcmp(utf8, header_test_expected) == 0);
+	g_assert(strcmp(utf8, test->expected) == 0);
 
 	g_free(utf8);
 }
@@ -685,7 +785,7 @@ static void test_assembly()
 	if (g_test_verbose())
 		g_printf("Text:\n%s\n", utf8);
 
-	l = sms_text_prepare(utf8, ref, TRUE, NULL);
+	l = sms_text_prepare(utf8, ref, TRUE, NULL, FALSE);
 	g_assert(l);
 	g_assert(g_slist_length(l) == 3);
 
@@ -715,7 +815,7 @@ static void test_prepare_7bit()
 	int encoded_tpdu_len;
 	char *encoded_pdu;
 
-	r = sms_text_prepare(test_no_fragmentation_7bit, 0, FALSE, NULL);
+	r = sms_text_prepare(test_no_fragmentation_7bit, 0, FALSE, NULL, FALSE);
 
 	g_assert(r != NULL);
 
@@ -762,16 +862,25 @@ static void test_prepare_7bit()
 	g_slist_free(r);
 }
 
-static const char *pad1 = "Shakespeare divided his time between London and Str"
+struct sms_concat_data {
+	const char *str;
+	unsigned int segments;
+};
+
+static struct sms_concat_data shakespeare_test = {
+	.str = "Shakespeare divided his time between London and Str"
 	"atford during his career. In 1596, the year before he bought New Plac"
 	"e as his family home in Stratford, Shakespeare was living in the pari"
-	"sh of St. Helen's, Bishopsgate, north of the River Thames.";
+	"sh of St. Helen's, Bishopsgate, north of the River Thames.",
+	.segments = 2,
+};
 
 /* The string in this test should be padded at the end.  This confuses some
  * decoders which do not use udl properly
  */
-static void test_prepare_concat()
+static void test_prepare_concat(gconstpointer data)
 {
+	const struct sms_concat_data *test = data;
 	GSList *r;
 	GSList *l;
 	char *decoded_str;
@@ -786,12 +895,11 @@ static void test_prepare_concat()
 	guint8 seq;
 
 	if (g_test_verbose())
-		g_print("strlen: %zd\n", strlen(pad1));
-
-	r = sms_text_prepare(pad1, 0, TRUE, NULL);
+		g_print("strlen: %zd\n", strlen(test->str));
 
+	r = sms_text_prepare(test->str, 0, TRUE, NULL, FALSE);
 	g_assert(r);
-	g_assert(g_slist_length(r) == 2);
+	g_assert(g_slist_length(r) == test->segments);
 
 	for (l = r; l; l = l->next) {
 		char *strpdu;
@@ -823,6 +931,7 @@ static void test_prepare_concat()
 			g_print("PDU Len: %ld\n", len);
 
 		ok = sms_decode(pdu, len, TRUE, len - 1, &decoded);
+		g_assert(ok);
 
 		if (g_test_verbose())
 			g_print("Pdu udl: %d\n", (int)decoded.submit.udl);
@@ -841,7 +950,7 @@ static void test_prepare_concat()
 		g_printf("Decoded String: %s\n", decoded_str);
 
 	g_assert(decoded_str);
-	g_assert(strcmp(decoded_str, pad1) == 0);
+	g_assert(strcmp(decoded_str, test->str) == 0);
 	g_free(decoded_str);
 	sms_assembly_free(assembly);
 }
@@ -864,7 +973,7 @@ static void test_limit(gunichar uni, int target_size, gboolean use_16bit)
 
 	utf8[i] = '\0';
 
-	l = sms_text_prepare(utf8, 0, use_16bit, NULL);
+	l = sms_text_prepare(utf8, 0, use_16bit, NULL, FALSE);
 
 	g_assert(l);
 	g_assert(g_slist_length(l) == 255);
@@ -877,7 +986,7 @@ static void test_limit(gunichar uni, int target_size, gboolean use_16bit)
 	memcpy(utf8 + i, utf8_char, stride);
 	utf8[i+stride] = '\0';
 
-	l = sms_text_prepare(utf8, 0, use_16bit, NULL);
+	l = sms_text_prepare(utf8, 0, use_16bit, NULL, FALSE);
 
 	g_assert(l == NULL);
 	g_free(utf8);
@@ -1158,8 +1267,192 @@ static void test_range_minimizer()
 	}
 }
 
+static void test_sr_assembly()
+{
+	const char *sr_pdu1 = "06040D91945152991136F00160124130340A0160124130"
+				"940A00";
+	const char *sr_pdu2 = "06050D91945152991136F00160124130640A0160124130"
+				"450A00";
+	const char *sr_pdu3 = "0606098121436587F9019012413064A0019012413045A0"
+				"00";
+        struct sms sr1;
+	struct sms sr2;
+	struct sms sr3;
+	unsigned char pdu[176];
+	long pdu_len;
+	struct status_report_assembly *sra;
+	gboolean delivered;
+	struct sms_address addr;
+	unsigned char sha1[SMS_MSGID_LEN] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
+						10, 11, 12, 13, 14, 15,
+						16, 17, 18, 19 };
+	unsigned char id[SMS_MSGID_LEN];
+
+	/* international address, mr 4 & mr 5 */
+
+        decode_hex_own_buf(sr_pdu1, -1, &pdu_len, 0, pdu);
+	g_assert(sms_decode(pdu, pdu_len, FALSE, 26, &sr1) == TRUE);
+
+	decode_hex_own_buf(sr_pdu2, -1, &pdu_len, 0, pdu);
+	g_assert(sms_decode(pdu, pdu_len, FALSE, 26, &sr2) == TRUE);
+
+	/* national address, mr 6 */
+
+	decode_hex_own_buf(sr_pdu3, -1, &pdu_len, 0, pdu);
+	g_assert(sms_decode(pdu, pdu_len, FALSE, 24, &sr3) == TRUE);
+
+	if (g_test_verbose()) {
+		g_print("sr1 address: %s, mr: %d\n",
+			sms_address_to_string(&sr1.status_report.raddr),
+			sr1.status_report.mr);
+
+		g_print("sr2 address: %s, mr: %d\n",
+			sms_address_to_string(&sr2.status_report.raddr),
+			sr2.status_report.mr);
+
+		g_print("sr3 address: %s, mr: %d\n",
+			sms_address_to_string(&sr3.status_report.raddr),
+			sr3.status_report.mr);
+	}
+
+	sms_address_from_string(&addr, "+4915259911630");
+
+	sra = status_report_assembly_new(NULL);
+
+	status_report_assembly_add_fragment(sra, sha1, &addr, 4, time(NULL), 2);
+	status_report_assembly_add_fragment(sra, sha1, &addr, 5, time(NULL), 2);
+
+	status_report_assembly_expire(sra, time(NULL) + 40);
+	g_assert(g_hash_table_size(sra->assembly_table) == 0);
+
+	status_report_assembly_add_fragment(sra, sha1, &addr, 4, time(NULL), 2);
+	status_report_assembly_add_fragment(sra, sha1, &addr, 5, time(NULL), 2);
+
+	g_assert(!status_report_assembly_report(sra, &sr1, id, &delivered));
+	g_assert(status_report_assembly_report(sra, &sr2, id, &delivered));
+
+	g_assert(memcmp(id, sha1, SMS_MSGID_LEN) == 0);
+	g_assert(delivered == TRUE);
+
+	/*
+	 * Send sms-message in the national address-format,
+	 * but receive in the international address-format.
+	 */
+	sms_address_from_string(&addr, "9911630");
+	status_report_assembly_add_fragment(sra, sha1, &addr, 4, time(NULL), 2);
+	status_report_assembly_add_fragment(sra, sha1, &addr, 5, time(NULL), 2);
+
+	g_assert(!status_report_assembly_report(sra, &sr1, id, &delivered));
+	g_assert(status_report_assembly_report(sra, &sr2, id, &delivered));
+
+	g_assert(memcmp(id, sha1, SMS_MSGID_LEN) == 0);
+	g_assert(delivered == TRUE);
+	g_assert(g_hash_table_size(sra->assembly_table) == 0);
+
+	/*
+	 * Send sms-message in the international address-format,
+	 * but receive in the national address-format.
+	 */
+	sms_address_from_string(&addr, "+358123456789");
+	status_report_assembly_add_fragment(sra, sha1, &addr, 6, time(NULL), 1);
+
+	g_assert(status_report_assembly_report(sra, &sr3, id, &delivered));
+
+	g_assert(memcmp(id, sha1, SMS_MSGID_LEN) == 0);
+	g_assert(delivered == TRUE);
+	g_assert(g_hash_table_size(sra->assembly_table) == 0);
+
+	status_report_assembly_free(sra);
+}
+
+struct wap_push_data {
+	const char *pdu;
+	int len;
+};
+
+static struct wap_push_data wap_push_1 = {
+	.pdu = "0791947122725014440185F039F501801140311480720605040B8423F00106"
+		"246170706C69636174696F6E2F766E642E7761702E6D6D732D6D657373616"
+		"76500AF84B4868C82984F67514B4B42008D9089088045726F74696B009650"
+		"696E2D557073008A808E0240008805810303F48083687474703A2F2F65707"
+		"3332E64652F4F2F5A39495A4F00",
+	.len = 128,
+};
+
+static void test_wap_push(gconstpointer data)
+{
+	const struct wap_push_data *test = data;
+	struct sms sms;
+	unsigned char *decoded_pdu;
+	gboolean ret;
+	long pdu_len;
+	long data_len;
+	enum sms_class cls;
+	enum sms_charset charset;
+	GSList *list;
+	unsigned char *wap_push;
+	int dst_port, src_port;
+	gboolean is_8bit;
+
+	decoded_pdu = decode_hex(test->pdu, -1, &pdu_len, 0);
+
+	g_assert(decoded_pdu);
+
+	ret = sms_decode(decoded_pdu, pdu_len, FALSE, test->len, &sms);
+
+	g_free(decoded_pdu);
+
+	g_assert(ret);
+	g_assert(sms.type == SMS_TYPE_DELIVER);
+
+	if (g_test_verbose())
+		dump_details(&sms);
+
+	ret = sms_dcs_decode(sms.deliver.dcs, &cls, &charset, NULL, NULL);
+
+	g_assert(ret == TRUE);
+	g_assert(charset == SMS_CHARSET_8BIT);
+
+	g_assert(sms_extract_app_port(&sms, &dst_port, &src_port, &is_8bit));
+
+	if (g_test_verbose()) {
+		g_print("8bit: %d\n", is_8bit);
+		g_print("src: %d, dst: %d\n", src_port, dst_port);
+	}
+
+	g_assert(is_8bit == FALSE);
+	g_assert(dst_port == 2948);
+
+	list = g_slist_append(NULL, &sms);
+
+	wap_push = sms_decode_datagram(list, &data_len);
+
+	if (g_test_verbose()) {
+		int i;
+
+		g_print("data_len: %ld\n", data_len);
+
+		for (i = 0; i < data_len; i++) {
+			g_print("%02x", wap_push[i]);
+
+			if ((i % 16) == 15)
+				g_print("\n");
+		}
+
+		g_print("\n");
+	}
+
+	g_assert(wap_push);
+
+	g_free(wap_push);
+	g_slist_free(list);
+}
+
 int main(int argc, char **argv)
 {
+	char long_string[152*33 + 1];
+	struct sms_concat_data long_string_test;
+
 	g_test_init(&argc, &argv, NULL);
 
 	g_test_add_func("/testsms/Test Simple Deliver", test_simple_deliver);
@@ -1167,10 +1460,30 @@ int main(int argc, char **argv)
 	g_test_add_func("/testsms/Test Deliver Encode", test_deliver_encode);
 	g_test_add_func("/testsms/Test Simple Submit", test_simple_submit);
 	g_test_add_func("/testsms/Test Submit Encode", test_submit_encode);
-	g_test_add_func("/testsms/Test UDH Iterator", test_udh_iter);
+
+	g_test_add_data_func("/testsms/Test EMS UDH 1",
+			&ems_udh_test_1, test_ems_udh);
+	g_test_add_data_func("/testsms/Test EMS UDH 2",
+			&ems_udh_test_2, test_ems_udh);
+
 	g_test_add_func("/testsms/Test Assembly", test_assembly);
 	g_test_add_func("/testsms/Test Prepare 7Bit", test_prepare_7bit);
-	g_test_add_func("/testsms/Test Prepare Concat", test_prepare_concat);
+
+	g_test_add_data_func("/testsms/Test Prepare Concat",
+			&shakespeare_test, test_prepare_concat);
+
+	memset(long_string, 'a', 152*30);
+	memset(long_string + 152*30, 'b', 152);
+	memset(long_string + 152*31, 'c', 152);
+	memset(long_string + 152*32, 'd', 152);
+	long_string[152*33] = '\0';
+
+	long_string_test.str = long_string;
+	long_string_test.segments = 33;
+
+	g_test_add_data_func("/testsms/Test Prepare Concat 30+ segments",
+			&long_string_test, test_prepare_concat);
+
 	g_test_add_func("/testsms/Test Prepare Limits", test_prepare_limits);
 
 	g_test_add_func("/testsms/Test CBS Encode / Decode",
@@ -1182,5 +1495,10 @@ int main(int argc, char **argv)
 
 	g_test_add_func("/testsms/Range minimizer", test_range_minimizer);
 
+	g_test_add_func("/testsms/Status Report Assembly", test_sr_assembly);
+
+	g_test_add_data_func("/testsms/Test WAP Push 1", &wap_push_1,
+				test_wap_push);
+
 	return g_test_run();
 }
